@@ -16,6 +16,8 @@
 from glanceclient.common import progressbar
 from glanceclient.common import utils
 from glanceclient import exc
+from glanceclient.v2.image_members import MEMBER_STATUS_VALUES
+from glanceclient.v2 import images
 from glanceclient.v2 import tasks
 import json
 import os
@@ -28,7 +30,7 @@ def get_image_schema():
     global IMAGE_SCHEMA
     if IMAGE_SCHEMA is None:
         schema_path = expanduser("~/.glanceclient/image_schema.json")
-        if os.path.exists(schema_path) and os.path.isfile(schema_path):
+        if os.path.isfile(schema_path):
             with open(schema_path, "r") as f:
                 schema_raw = f.read()
                 IMAGE_SCHEMA = json.loads(schema_raw)
@@ -42,9 +44,9 @@ def get_image_schema():
            default=[], help=('Arbitrary property to associate with image.'
                              ' May be used multiple times.'))
 @utils.arg('--file', metavar='<FILE>',
-           help='Local file to save downloaded image data to. '
-                'If this is not specified the image data will be '
-                'written to stdout.')
+           help='Local file that contains disk image to be uploaded '
+                'during creation. Alternatively, images can be passed '
+                'to the client via stdin.')
 @utils.arg('--progress', action='store_true', default=False,
            help='Show upload progress bar.')
 def do_image_create(gc, args):
@@ -67,7 +69,7 @@ def do_image_create(gc, args):
                    "privileges to it" % file_name)
     image = gc.images.create(**fields)
     try:
-        if file_name is not None:
+        if utils.get_data_file(args) is not None:
             args.id = image['id']
             args.size = None
             do_image_upload(gc, args)
@@ -124,6 +126,16 @@ def do_image_update(gc, args):
            help='Displays images that match the checksum.')
 @utils.arg('--tag', metavar='<TAG>', action='append',
            help="Filter images by a user-defined tag.")
+@utils.arg('--sort-key', default=[], action='append',
+           choices=images.SORT_KEY_VALUES,
+           help='Sort image list by specified fields.')
+@utils.arg('--sort-dir', default=[], action='append',
+           choices=images.SORT_DIR_VALUES,
+           help='Sort image list in specified directions.')
+@utils.arg('--sort', metavar='<key>[:<direction>]', default=None,
+           help=(("Comma-separated list of sort keys and directions in the "
+                  "form of <key>[:<asc|desc>]. Valid keys: %s. OPTIONAL: "
+                  "Default='name:asc'.") % ', '.join(images.SORT_KEY_VALUES)))
 def do_image_list(gc, args):
     """List images you can access."""
     filter_keys = ['visibility', 'member_status', 'owner', 'checksum', 'tag']
@@ -138,10 +150,19 @@ def do_image_list(gc, args):
 
     kwargs = {'filters': filters}
     if args.limit is not None:
-        kwargs['limit'] = args.page_size
+        kwargs['limit'] = args.limit
     if args.page_size is not None:
         kwargs['page_size'] = args.page_size
 
+    if args.sort_key:
+        kwargs['sort_key'] = args.sort_key
+    if args.sort_dir:
+        kwargs['sort_dir'] = args.sort_dir
+    if args.sort is not None:
+        kwargs['sort'] = args.sort
+    elif not args.sort_dir and not args.sort_key:
+        kwargs['sort'] = 'name:asc'
+
     images = gc.images.list(**kwargs)
     columns = ['ID', 'Name']
     utils.print_list(images, columns)
@@ -183,7 +204,10 @@ def do_member_delete(gc, args):
 @utils.arg('member_id', metavar='<MEMBER_ID>',
            help='Tenant to update.')
 @utils.arg('member_status', metavar='<MEMBER_STATUS>',
-           help='Updated status of member.')
+           choices=MEMBER_STATUS_VALUES,
+           help='Updated status of member.'
+                ' Valid Values: %s' %
+                ', '.join(str(val) for val in MEMBER_STATUS_VALUES))
 def do_member_update(gc, args):
     """Update the status of a member for a given image."""
     if not (args.image_id and args.member_id and args.member_status):
@@ -356,7 +380,7 @@ def get_namespace_schema():
     global NAMESPACE_SCHEMA
     if NAMESPACE_SCHEMA is None:
         schema_path = expanduser("~/.glanceclient/namespace_schema.json")
-        if os.path.exists(schema_path) and os.path.isfile(schema_path):
+        if os.path.isfile(schema_path):
             with open(schema_path, "r") as f:
                 schema_raw = f.read()
                 NAMESPACE_SCHEMA = json.loads(schema_raw)
@@ -495,7 +519,7 @@ def get_resource_type_schema():
     global RESOURCE_TYPE_SCHEMA
     if RESOURCE_TYPE_SCHEMA is None:
         schema_path = expanduser("~/.glanceclient/resource_type_schema.json")
-        if os.path.exists(schema_path) and os.path.isfile(schema_path):
+        if os.path.isfile(schema_path):
             with open(schema_path, "r") as f:
                 schema_raw = f.read()
                 RESOURCE_TYPE_SCHEMA = json.loads(schema_raw)
